@@ -14,12 +14,14 @@
 # under the License.
 
 import collections
+import os
 import time
 import uuid
 
 import fixtures
-import os
+from keystoneauth1 import loading as ks_loading
 import openstack.config as occ
+from oslo_config import cfg
 from requests import structures
 from requests_mock.contrib import fixture as rm_fixture
 from six.moves import urllib
@@ -82,6 +84,8 @@ _RoleData = collections.namedtuple(
 
 class TestCase(base.TestCase):
 
+    strict_cloud = False
+
     def setUp(self, cloud_config_fixture='clouds.yaml'):
         """Run before each test method to initialize test environment."""
 
@@ -111,19 +115,27 @@ class TestCase(base.TestCase):
         vendor.write(b'{}')
         vendor.close()
 
-        test_cloud = os.environ.get('OPENSTACKSDK_OS_CLOUD', '_test_cloud_')
         self.config = occ.OpenStackConfig(
             config_files=[config.name],
             vendor_files=[vendor.name],
             secure_files=['non-existant'])
-        self.cloud_config = self.config.get_one(
-            cloud=test_cloud, validate=False)
-        self.cloud = openstack.connection.Connection(
-            config=self.cloud_config,
-            strict=False)
-        self.strict_cloud = openstack.connection.Connection(
-            config=self.cloud_config,
-            strict=True)
+
+        self.oslo_config_dict = {
+            # All defaults for nova
+            'nova': {},
+            # monasca-api not in the service catalog
+            'monasca-api': {},
+            # Overrides for heat
+            'heat': {
+                'region_name': 'SpecialRegion',
+                'interface': 'internal',
+                'endpoint_override': 'https://example.org:8888/heat/v2'
+            },
+            # test a service with dashes
+            'ironic_inspector': {
+                'endpoint_override': 'https://example.org:5050',
+            },
+        }
 
         # FIXME(notmorgan): Convert the uri_registry, discovery.json, and
         # use of keystone_v3/v2 to a proper fixtures.Fixture. For now this
@@ -146,6 +158,16 @@ class TestCase(base.TestCase):
         self.use_keystone_v3()
         self.__register_uris_called = False
 
+    def _load_ks_cfg_opts(self):
+        conf = cfg.ConfigOpts()
+        for group, opts in self.oslo_config_dict.items():
+            conf.register_group(cfg.OptGroup(group))
+            if opts is not None:
+                ks_loading.register_adapter_conf_options(conf, group)
+                for name, val in opts.items():
+                    conf.set_override(name, val, group=group)
+        return conf
+
     # TODO(shade) Update this to handle service type aliases
     def get_mock_url(self, service_type, interface='public', resource=None,
                      append=None, base_url_append=None,
@@ -398,24 +420,34 @@ class TestCase(base.TestCase):
         self.calls = []
         self._uri_registry.clear()
 
+    def get_keystone_v3_token(self, catalog='catalog-v3.json'):
+        catalog_file = os.path.join(self.fixtures_directory, catalog)
+        with open(catalog_file, 'r') as tokens_file:
+            return dict(
+                method='POST',
+                uri='https://identity.example.com/v3/auth/tokens',
+                headers={
+                    'X-Subject-Token': self.getUniqueString('KeystoneToken')
+                },
+                text=tokens_file.read()
+            )
+
+    def get_keystone_v3_discovery(self):
+        with open(self.discovery_json, 'r') as discovery_file:
+            return dict(
+                method='GET',
+                uri='https://identity.example.com/',
+                text=discovery_file.read(),
+            )
+
     def use_keystone_v3(self, catalog='catalog-v3.json'):
         self.adapter = self.useFixture(rm_fixture.Fixture())
         self.calls = []
         self._uri_registry.clear()
-        with open(self.discovery_json, 'r') as discovery_file, \
-            open(os.path.join(
-                self.fixtures_directory, catalog), 'r') as tokens_file:
-            self.__do_register_uris([
-                dict(method='GET', uri='https://identity.example.com/',
-                     text=discovery_file.read()),
-                dict(method='POST',
-                     uri='https://identity.example.com/v3/auth/tokens',
-                     headers={
-                         'X-Subject-Token':
-                         self.getUniqueString('KeystoneToken')},
-                     text=tokens_file.read()
-                     ),
-            ])
+        self.__do_register_uris([
+            self.get_keystone_v3_discovery(),
+            self.get_keystone_v3_token(catalog),
+        ])
         self._make_test_cloud(identity_api_version='3')
 
     def use_keystone_v2(self):
@@ -443,9 +475,17 @@ class TestCase(base.TestCase):
         test_cloud = os.environ.get('OPENSTACKSDK_OS_CLOUD', cloud_name)
         self.cloud_config = self.config.get_one(
             cloud=test_cloud, validate=True, **kwargs)
-        self.conn = openstack.connection.Connection(
-            config=self.cloud_config)
-        self.cloud = self.conn
+        self.cloud = openstack.connection.Connection(
+            config=self.cloud_config, strict=self.strict_cloud)
+
+    def get_cinder_discovery_mock_dict(
+            self,
+            block_storage_version_json='block-storage-version.json',
+            block_storage_discovery_url='https://volume.example.com/'):
+        discovery_fixture = os.path.join(
+            self.fixtures_directory, block_storage_version_json)
+        return dict(method='GET', uri=block_storage_discovery_url,
+                    text=open(discovery_fixture, 'r').read())
 
     def get_glance_discovery_mock_dict(
             self,
