@@ -56,11 +56,41 @@ class TestShade(base.TestCase):
 
     def test_connect_as(self):
         # Do initial auth/catalog steps
-        # TODO(mordred) This only tests the constructor steps. Discovery
-        # cache sharing is broken. We need to get discovery_cache option
-        # plumbed through
-        # keystoneauth1.loading.base.BaseLoader.load_from_options
-        self.cloud.connect_as(project_name='test_project')
+        # This should authenticate a second time, but should not
+        # need a second identity discovery
+        self.register_uris([
+            self.get_keystone_v3_token(),
+            self.get_nova_discovery_mock_dict(),
+            dict(
+                method='GET',
+                uri=self.get_mock_url(
+                    'compute', 'public', append=['servers', 'detail']),
+                json={'servers': []},
+            ),
+        ])
+
+        c2 = self.cloud.connect_as(project_name='test_project')
+        self.assertEqual(c2.list_servers(), [])
+        self.assert_calls()
+
+    def test_connect_as_context(self):
+        # Do initial auth/catalog steps
+        # This should authenticate a second time, but should not
+        # need a second identity discovery
+        self.register_uris([
+            self.get_keystone_v3_token(),
+            self.get_nova_discovery_mock_dict(),
+            dict(
+                method='GET',
+                uri=self.get_mock_url(
+                    'compute', 'public', append=['servers', 'detail']),
+                json={'servers': []},
+            ),
+        ])
+
+        with self.cloud.connect_as(project_name='test_project') as c2:
+            self.assertEqual(c2.list_servers(), [])
+        self.assert_calls()
 
     @mock.patch.object(connection.Connection, 'search_images')
     def test_get_images(self, mock_search):
@@ -76,6 +106,44 @@ class TestShade(base.TestCase):
         r = self.cloud.get_image('doesNotExist')
         self.assertIsNone(r)
 
+    def test_global_request_id(self):
+        request_id = uuid.uuid4().hex
+        self.register_uris([
+            self.get_nova_discovery_mock_dict(),
+            dict(
+                method='GET',
+                uri=self.get_mock_url(
+                    'compute', 'public', append=['servers', 'detail']),
+                json={'servers': []},
+                validate=dict(
+                    headers={'X-Openstack-Request-Id': request_id}),
+            ),
+        ])
+
+        cloud2 = self.cloud.global_request(request_id)
+        self.assertEqual([], cloud2.list_servers())
+
+        self.assert_calls()
+
+    def test_global_request_id_context(self):
+        request_id = uuid.uuid4().hex
+        self.register_uris([
+            self.get_nova_discovery_mock_dict(),
+            dict(
+                method='GET',
+                uri=self.get_mock_url(
+                    'compute', 'public', append=['servers', 'detail']),
+                json={'servers': []},
+                validate=dict(
+                    headers={'X-Openstack-Request-Id': request_id}),
+            ),
+        ])
+
+        with self.cloud.global_request(request_id) as c2:
+            self.assertEqual([], c2.list_servers())
+
+        self.assert_calls()
+
     def test_get_server(self):
         server1 = fakes.make_fake_server('123', 'mickey')
         server2 = fakes.make_fake_server('345', 'mouse')
