@@ -15,9 +15,12 @@ import os
 import fixtures
 from keystoneauth1 import session
 import mock
+from testtools import matchers
 
 from openstack import connection
+from openstack import proxy
 import openstack.config
+from openstack import service_description
 from openstack.tests.unit import base
 from openstack.tests.unit.fake import fake_service
 
@@ -91,10 +94,10 @@ public-clouds:
 """.format(auth_url=CONFIG_AUTH_URL)
 
 
-class TestConnection(base.TestCase):
+class _TestConnectionBase(base.TestCase):
 
     def setUp(self):
-        super(TestConnection, self).setUp()
+        super(_TestConnectionBase, self).setUp()
         # Create a temporary directory where our test config will live
         # and insert it into the search path via OS_CLIENT_CONFIG_FILE.
         config_dir = self.useFixture(fixtures.TempDir()).path
@@ -107,6 +110,8 @@ class TestConnection(base.TestCase):
             "OS_CLIENT_CONFIG_FILE", config_path))
         self.use_keystone_v2()
 
+
+class TestConnection(_TestConnectionBase):
     def test_other_parameters(self):
         conn = connection.Connection(cloud='sample-cloud', cert='cert')
         self.assertEqual(conn.session.cert, 'cert')
@@ -145,6 +150,24 @@ class TestConnection(base.TestCase):
         # self.assertEqual('openstack.workflow.v2._proxy',
         #                  conn.workflow.__class__.__module__)
 
+    def test_create_unknown_proxy(self):
+        self.register_uris([
+            self.get_placement_discovery_mock_dict(),
+        ])
+
+        def closure():
+            return self.cloud.placement
+
+        self.assertThat(
+            closure,
+            matchers.Warnings(matchers.HasLength(0)))
+
+        self.assertIsInstance(
+            self.cloud.placement,
+            proxy.Proxy)
+
+        self.assert_calls()
+
     def test_create_connection_version_param_default(self):
         c1 = connection.Connection(cloud='sample-cloud')
         conn = connection.Connection(session=c1.session)
@@ -240,6 +263,31 @@ class TestConnection(base.TestCase):
         self.assertFalse(sot.session.verify)
 
 
+class TestOsloConfig(_TestConnectionBase):
+    def test_from_conf(self):
+        c1 = connection.Connection(cloud='sample-cloud')
+        conn = connection.Connection(
+            session=c1.session, oslo_conf=self._load_ks_cfg_opts())
+        # There was no config for keystone
+        self.assertIsInstance(
+            conn.identity, service_description._ServiceDisabledProxyShim)
+        # But nova was in there
+        self.assertEqual('openstack.compute.v2._proxy',
+                         conn.compute.__class__.__module__)
+
+    def test_from_conf_filter_service_types(self):
+        c1 = connection.Connection(cloud='sample-cloud')
+        conn = connection.Connection(
+            session=c1.session, oslo_conf=self._load_ks_cfg_opts(),
+            service_types={'orchestration', 'i-am-ignored'})
+        # There was no config for keystone
+        self.assertIsInstance(
+            conn.identity, service_description._ServiceDisabledProxyShim)
+        # Nova was in there, but disabled because not requested
+        self.assertIsInstance(
+            conn.compute, service_description._ServiceDisabledProxyShim)
+
+
 class TestNetworkConnection(base.TestCase):
 
     # Verify that if the catalog has the suffix we don't mess things up.
